
#include <avr/io.h>
#include <util/delay.h>
#include "ili9327.h"

const uint16_t BLACK    = 0x0000;
const uint16_t BLUE     = 0x001F;
const uint16_t RED      = 0xF800;
const uint16_t GREEN    = 0x07E0;
const uint16_t CYAN     = 0x07FF;
const uint16_t MAGENTA  = 0xF81F;
const uint16_t YELLOW   = 0xFFE0;
const uint16_t WHITE    = 0xFFFF;

static uint16_t const WIDTH = 240;
static uint16_t const LENGTH = 400;

//==============================================================================
void displayColorBars(void)
{
  uint16_t i,j;

  //Always return home before writing to the screen
  displayHome();

  for(i = 0; i < WIDTH; i++)
  {
    for(j = 0; j < LENGTH; j++)
    {
      if     (i>209)  writeColor(BLACK);
      else if(i>179)  writeColor(BLUE);
      else if(i>149)  writeColor(RED);
      else if(i>119)  writeColor(GREEN);
      else if(i>89)   writeColor(CYAN);
      else if(i>59)   writeColor(MAGENTA);
      else if(i>29)   writeColor(YELLOW);
      else            writeColor(WHITE);
    }
  }
}
//==============================================================================
void displayHome(void)
{
  //This function is Hard coded 240x400

  //Set the display width
  writeCommand(ILI9327_SET_COL_ADDR); //0-239 == 0x0000-0x00EF
  writeData(0x00);  //SC8
  writeData(0x00);  //SC7-SC0
  writeData(0x00);  //EC8
  writeData(0xEF);  //EC7-EC0

  //Set the display length
  writeCommand(ILI9327_SET_PAGE_ADDR); //0-399 == 0x0000-0x018F
  writeData(0x00);  //SP8
  writeData(0x00);  //SP7-SP0
  writeData(0x01);  //EP8
  writeData(0x8F);  //EP7-EP0

  writeCommand(ILI9327_WR_MEM_START);
}
//==============================================================================
void displayFillColor(uint16_t data)
{
  uint16_t i,j;

  //Always return home before writing to the screen
  displayHome();

  for(i = 0; i < LENGTH; i++)
  {
    for(j = 0; j < WIDTH; j++)
    {
			writeColor(data);
		}
	}
}
//==============================================================================
void enterStandby(void)
{
  writeCommand(ILI9327_DISPLAY_OFF);
  _delay_ms(10); 
  writeCommand(ILI9327_ENTER_SLEEP);
  _delay_ms(25); 
}
//==============================================================================
void exitStandby (void)
{
  writeCommand(ILI9327_EXIT_SLEEP);
  _delay_ms(120);
  writeCommand(ILI9327_DISPLAY_ON);
}
//==============================================================================
void writeColor(uint16_t data)
{
  
	CLR_CS; //Assert chip select
	SET_CD; //Select data register
	CLR_WR; //Assert write mode

  HIGH_BYTE = data>>8;
  LOW_BYTE = data;

	SET_WR; //De-assert write mode indicating data is "ready" to be read
	SET_CS; //De-assert chip select
}
//==============================================================================
void writeCommand(uint8_t command)
{
  CLR_CS; //Assert chip select
  CLR_CD; //Select command register
  CLR_WR; //Assert write mode

  HIGH_BYTE=0x00;
  LOW_BYTE=command;

  SET_WR; //De-assert write mode indicating command is "ready" to be read
  SET_CS; //De-assert chip select
}
//==============================================================================
void writeData(uint8_t data)
{
	CLR_CS; //Assert chip select
	SET_CD; //Select data register
	CLR_WR; //Assert write mode

  HIGH_BYTE=0x00;
  LOW_BYTE=data;

	SET_WR; //De-assert write mode indicating data is "ready" to be read
	SET_CS; //De-assert chip select
}
//==============================================================================
